@@ -2,25 +2,29 @@
 /*** KMPW1 TRANSITION PROBABILITIES ***/
 /*************************************/
 
+#ifdef _OPENMP
 #include <omp.h>
+#endif
 #include <stdlib.h>
 #include <time.h>
 #include <Rdefines.h>
 #include "defines.h"
 #include "boot.h"
 #include "get.h"
+#include "rthreads.h"
 #include "sort.h"
 #include "logistic.h"
 
 /*
 Author:
-	Artur Agostinho Arajo <b5498@math.uminho.pt>
+	Artur Agostinho Araujo <artur.stat@gmail.com>
 
 Description:
 	Computes the transition probabilities:
-		p11(s,t) = P(Z>t|Z>s) = P(Z>t)/P(Z>s)
-		p12(s,t) = P(Z<=t,T>t|Z>s) = P(s<Z<=t,T>t)/P(Z>s)
-		p22(s,t) = P(Z<=t,T>t|Z<=s,T>s) = P(Z<=s,T>t)/P(Z<=s,T>s)
+		p11(s,t) = P(Z>t|Z>s) = {1-P(Z<=t)}/{1-P(Z<=s)}
+		p12(s,t) = P(Z<=t,T>t|Z>s) = {P(Z<=t)-P(Z<=s)-P(s<Z<=t,T<=t)}/{1-P(Z<=s)}
+		p13(s,t) = 1-p11(s,t)-p12(s,t)
+		p22(s,t) = P(Z<=t,T>t|Z<=s,T>s) = {P(Z<=s)-P(Z<=s,T<=t)}/{P(Z<=s)-P(T<=s)}
 
 Parameters:
 	len[in]			pointer to length of T1, M0, S and MS.
@@ -33,7 +37,7 @@ Parameters:
 	nt[in]			pointer to length of UT and number of rows of P.
 	UT[in]			pointer to unique times vector.
 	nb[in]			pointer to number of rows of P.
-	P[out]			pointer to a (nb)x(nt)x3 probability array.
+	P[out]			pointer to a (nb)x(nt)x4 probability array.
 	b[in]			pointer to row index.
 
 Return value:
@@ -58,7 +62,7 @@ static void transKMPW1I(
 	CintCP nt,
 	Cdouble UT[*nt],
 	CintCP nb,
-	double P[*nb*(*nt)*3],
+	double P[*nb*(*nt)*4],
 	CintCP b)
 {
 	register int i;
@@ -77,9 +81,10 @@ static void transKMPW1I(
 		p[0] = aux[4]/aux[3]; // compute transition probability
 		p[1] = 1-p[0]; // compute probability
 		while (T1[index0[j]] > UT[i]) {
-			P[*b+*nb*i] = p[0]; // save transition probability
+			if (p[0] < 0) P[*b+*nb*i] = 0;
+			else P[*b+*nb*i] = p[0]; // save p11(s,t)
 			P[*b+*nb*(i+*nt)] = p[1]; // save probability
-			P[*b+*nb*(i+*nt*2)] = 1;
+			P[*b+*nb*(i+*nt*3)] = 1;
 			i++;
 		}
 		aux[2] = M0[index0[j]]/(*len-j); // compute needed factor
@@ -90,10 +95,11 @@ static void transKMPW1I(
 	}
 	p[0] = aux[4]/aux[3]; // compute and save transition probability
 	p[1] = 1-p[0]; // compute and save probability
+	if (p[0] < 0) p[0] = 0;
 	for (; i < *nt; i++) {
-		P[*b+*nb*i] = p[0];
-		P[*b+*nb*(i+*nt)] = p[1];
-		P[*b+*nb*(i+*nt*2)] = 1;
+		P[*b+*nb*i] = p[0]; // save p11(s,t)
+		P[*b+*nb*(i+*nt)] = p[1]; // save probability
+		P[*b+*nb*(i+*nt*3)] = 1;
 	}
 	j = 0;
 	getIndexI(S, index1, &UT[0], len, &j, &e); // determine first index
@@ -109,8 +115,15 @@ static void transKMPW1I(
 		p[0] = aux[5]/aux[3]; // compute probability
 		p[1] = aux[6]/(1-aux[3]-aux[4]); // compute probability
 		while (S[index1[j]] > UT[i]) {
-			P[*b+*nb*(i+*nt)] -= p[0]; // compute and save transition probability
-			P[*b+*nb*(i+*nt*2)] -= p[1]; // compute and save transition probability
+			P[*b+*nb*(i+*nt)] -= p[0]; // compute and save p12(s,t)
+			if (P[*b+*nb*(i+*nt)] < 0) P[*b+*nb*(i+*nt)] = 0;
+			P[*b+*nb*(i+*nt*2)] = 1-P[*b+*nb*i]-P[*b+*nb*(i+*nt)]; // compute and save p13(s,t)
+			if (P[*b+*nb*(i+*nt*2)] < 0) {
+				P[*b+*nb*(i+*nt)] = 1-P[*b+*nb*i]; // compute and save p12(s,t)
+				P[*b+*nb*(i+*nt*2)] = 0; // compute and save p13(s,t)
+			}
+			P[*b+*nb*(i+*nt*3)] -= p[1]; // compute and save p22(s,t)
+			if (P[*b+*nb*(i+*nt*3)] < 0) P[*b+*nb*(i+*nt*3)] = 0;
 			i++;
 		}
 		aux[2] = MS[index1[j]]/(*len-j); // compute needed factor
@@ -123,20 +136,28 @@ static void transKMPW1I(
 	p[0] = aux[5]/aux[3]; // compute probability
 	p[1] = aux[6]/(1-aux[3]-aux[4]); // compute probability
 	for (; i < *nt; i++) {
-		P[*b+*nb*(i+*nt)] -= p[0]; // compute and save transition probability
-		P[*b+*nb*(i+*nt*2)] -= p[1]; // compute and save transition probability
+		P[*b+*nb*(i+*nt)] -= p[0]; // compute and save p12(s,t)
+		if (P[*b+*nb*(i+*nt)] < 0) P[*b+*nb*(i+*nt)] = 0;
+		P[*b+*nb*(i+*nt*2)] = 1-P[*b+*nb*i]-P[*b+*nb*(i+*nt)]; // compute and save p13(s,t)
+		if (P[*b+*nb*(i+*nt*2)] < 0) {
+			P[*b+*nb*(i+*nt)] = 1-P[*b+*nb*i]; // compute and save p12(s,t)
+			P[*b+*nb*(i+*nt*2)] = 0; // compute and save p13(s,t)
+		}
+		P[*b+*nb*(i+*nt*3)] -= p[1]; // compute and save p22(s,t)
+		if (P[*b+*nb*(i+*nt*3)] < 0) P[*b+*nb*(i+*nt*3)] = 0;
 	}
 	return;
 } // transKMPW1I
 
 /*
 Author:
-	Artur Agostinho Arajo <b5498@math.uminho.pt>
+	Artur Agostinho Araujo <artur.stat@gmail.com>
 
 Description:
 	Computes the transition probabilities:
 		p11(s,t) = P(Z>t|Z>s) = P(Z>t)/P(Z>s)
 		p12(s,t) = P(Z<=t,T>t|Z>s) = P(s<Z<=t,T>t)/P(Z>s)
+		p13(s,t) = 1-p11(s,t)-p12(s,t)
 		p22(s,t) = P(Z<=t,T>t|Z<=s,T>s) = P(Z<=s,T>t)/P(Z<=s,T>s)
 
 Parameters:
@@ -150,7 +171,7 @@ Parameters:
 	nt[in]			pointer to length of UT and number of rows of P.
 	UT[in]			pointer to unique times vector.
 	nb[in]			pointer to number of rows of P.
-	P[out]			pointer to a (nb)x(nt)x3 probability array.
+	P[out]			pointer to a (nb)x(nt)x4 probability array.
 	b[in]			pointer to row index.
 
 Return value:
@@ -175,7 +196,7 @@ static void transKMPW2I(
 	CintCP nt,
 	Cdouble UT[*nt],
 	CintCP nb,
-	double P[*nb*(*nt)*3],
+	double P[*nb*(*nt)*4],
 	CintCP b)
 {
 	register int i, k;
@@ -192,9 +213,9 @@ static void transKMPW2I(
 	getIndexI(T1, index0, &UT[*nt-1], len, &j, &e); // determine last index
 	for (i = 0; j < e; j++) { // loop through the sample until last index is reached
 		while (T1[index0[j]] > UT[i]) {
-			P[*b+*nb*i] = aux[3]; // save transition probability
+			P[*b+*nb*i] = aux[3];
 			P[*b+*nb*(i+*nt)] = 0;
-			P[*b+*nb*(i+*nt*2)] = 0;
+			P[*b+*nb*(i+*nt*3)] = 0;
 			i++;
 		}
 		aux[2] = M0[index0[j]]/(*len-j); // compute needed factor
@@ -206,7 +227,7 @@ static void transKMPW2I(
 	for (; i < *nt; i++) {
 		P[*b+*nb*i] = aux[3];
 		P[*b+*nb*(i+*nt)] = 0;
-		P[*b+*nb*(i+*nt*2)] = 0;
+		P[*b+*nb*(i+*nt*3)] = 0;
 	}
 	for (; j < *len; j++) { // loop through the sample until last index is reached
 		aux[2] = M0[index0[j]]/(*len-j); // compute needed factor
@@ -229,7 +250,7 @@ static void transKMPW2I(
 		aux[2] *= aux[0]; // compute and save weight
 		aux[0] *= aux[1]; // compute and save factor needed for next iteration
 		while (S[index1[j]] > UT[k]) k++;
-		if (T1[index1[j]] <= UT[0]) for (i = 0; i < k; i++) P[*b+*nb*(i+*nt*2)] += aux[2];
+		if (T1[index1[j]] <= UT[0]) for (i = 0; i < k; i++) P[*b+*nb*(i+*nt*3)] += aux[2];
 		else for (i = 0; i < k; i++) P[*b+*nb*(i+*nt)] += aux[2]*(T1[index1[j]] <= UT[i]);
 	}
 	for (; j < *len; j++) { // loop through the sample until last index is reached
@@ -237,20 +258,25 @@ static void transKMPW2I(
 		aux[1] = 1-aux[2]; // factor needed for the computation
 		aux[2] *= aux[0]; // compute and save weight
 		aux[0] *= aux[1]; // compute and save factor needed for next iteration
-		if (T1[index1[j]] <= UT[0]) for (i = 0; i < *nt; i++) P[*b+*nb*(i+*nt*2)] += aux[2];
+		if (T1[index1[j]] <= UT[0]) for (i = 0; i < *nt; i++) P[*b+*nb*(i+*nt*3)] += aux[2];
 		else for (i = 0; i < *nt; i++) P[*b+*nb*(i+*nt)] += aux[2]*(T1[index1[j]] <= UT[i]);
 	}
 	for (i = *nt-1; i >= 0; i--) {
-		P[*b+*nb*(i+*nt)] /= P[*b];
-		P[*b+*nb*i] /= P[*b];
-		P[*b+*nb*(i+*nt*2)] /= P[*b+*nb*(*nt*2)];
+		P[*b+*nb*(i+*nt)] /= P[*b]; // compute and save p12(s,t)
+		P[*b+*nb*i] /= P[*b]; // compute and save p11(s,t)
+		P[*b+*nb*(i+*nt*2)] = 1-P[*b+*nb*i]-P[*b+*nb*(i+*nt)]; // compute and save p13(s,t)
+		if (P[*b+*nb*(i+*nt*2)] < 0) {
+			P[*b+*nb*(i+*nt)] = 1-P[*b+*nb*i]; // compute and save p12(s,t)
+			P[*b+*nb*(i+*nt*2)] = 0; // compute and save p13(s,t)
+		}
+		P[*b+*nb*(i+*nt*3)] /= P[*b+*nb*(*nt*3)]; // compute and save p22(s,t)
 	}
 	return;
 } // transKMPW2I
 
 /*
 Author:
-	Artur Agostinho Arajo <b5498@math.uminho.pt>
+	Artur Agostinho Araujo <artur.stat@gmail.com>
 
 Description:
 	Computes a transition probability vector based
@@ -264,7 +290,7 @@ Parameters:
 
 Return value:
 	Returns a list where the first element is a
-		(nboot)x(nt)x3 array of transition probabilities,
+		(nboot)x(nt)x4 array of transition probabilities,
 		and the second element is NULL.
 */
 
@@ -282,7 +308,7 @@ SEXP TransPROBKMPW1(
 	E = VECTOR_ELT(data, 3);
 	int len = GET_LENGTH(T1), nt = GET_LENGTH(UT);
 	SEXP P, list;
-	PROTECT( P = alloc3DArray(REALSXP, *INTEGER(nboot), nt, 3) );
+	PROTECT( P = alloc3DArray(REALSXP, *INTEGER(nboot), nt, 4) );
 	PROTECT( list = NEW_LIST(2) );
 	void (*func)(CintCP, CdoubleCP, CdoubleCP, CdoubleCP, CdoubleCP, CintCP, CintCP, CintCP, CdoubleCP, CintCP, doubleCP, CintCP);
 	switch ( *INTEGER(methodest) ) {
@@ -299,7 +325,7 @@ SEXP TransPROBKMPW1(
 	double epsilon = 1e-8;
 	for (b = 0; b < len; b++) J[b] = 1; // initialize J vector
 	#ifdef _OPENMP
-	#pragma omp parallel if(*INTEGER(nboot) > 1) private(b)
+	#pragma omp parallel if(*INTEGER(nboot) > 1) num_threads(global_num_threads) private(b)
 	#endif
 	{
 		int conv;
